/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.examples.reports.availability;

import com.net2plan.interfaces.networkDesign.Net2PlanException;
import com.net2plan.interfaces.networkDesign.NetPlan;
import com.net2plan.libraries.SRGUtils;
import com.net2plan.utils.*;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.*;

/**
 * This report analyzes a network design in terms of average traffic survivability
 * under a set of disaster failures.
 *
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @version 1.1, May 2015
 */
public class Report_disasterVulnerability extends Report_availability
{
	@Override
	protected List<Set<Long>> defineFailureStates(Set<Long> srgIds, boolean analyzeDoubleFailures)
	{
		return SRGUtils.enumerateFailureStates(srgIds, false, false);
	}

	@Override
	protected List<Double> defineStateProbabilities(List<Set<Long>> F_s, Map<Long, Double> A_f)
	{
		if (F_s.isEmpty()) throw new Net2PlanException("No failure states");

		int S = F_s.size();
		double pi_s_anyState = 1.0/S;
		List<Double> pi_s = new LinkedList<Double>();
		for(int i = 0; i < S; i++) pi_s.add(pi_s_anyState);
		
		return pi_s;
	}

	@Override
	public String executeReport(NetPlan netPlan, Map<String, String> reportParameters, Map<String, String> net2planParameters)
	{
		reportParameters.put("analyzeDoubleFailures", "false");

		return super.executeReport(netPlan, reportParameters, net2planParameters);
	}

	@Override
	public String getDescription()
	{
		return "This report analyzes a network design in terms of average traffic survivability under a set of disaster failures";
	}

	@Override
	public List<Triple<String, String, String>> getParameters()
	{
		List<Triple<String, String, String>> reportParameters = super.getParameters();

		Iterator<Triple<String, String, String>> it = reportParameters.iterator();

		String paramToRemove = "analyzeDoubleFailures";
		while (it.hasNext())
			if (it.next().getFirst().equals(paramToRemove))
				it.remove();

		return reportParameters;
	}
	
	@Override
	protected URL getReportTemplateFile()
	{
		try { return Report_disasterVulnerability.class.getResource("/com/net2plan/examples/reports/availability/Report_disasterVulnerability.xsl").toURI().toURL(); }
		catch (URISyntaxException | MalformedURLException ex) { throw new RuntimeException(ex); }
	}

	@Override
	public String getTitle()
	{
		return "Disaster-vulnerability report";
	}
}